@@ -1,6 +1,6 @@
-from rpi2mqtt.switch import BasicSwitch
+from rpi2mqtt.switch import BasicSwitch, Switch
 from rpi2mqtt.base import Sensor
-import rpi2mqtt.mqtt as mqtt
+from rpi2mqtt.mqtt import MQTT
 from rpi2mqtt.temperature import BME280
 import RPi.GPIO as GPIO
 import pendulum
@@ -21,7 +21,7 @@ class HVAC(object):
         'off': [],
         'fan': [HEAT_PUMP['fan']],
         'heat': [HEAT_PUMP['fan'], HEAT_PUMP['compressor']],
-        'cool': [HEAT_PUMP['fan'], HEAT_PUMP['compressor'], HEAT_PUMP['reversing_valve']],
+        'cool': [HEAT_PUMP['reversing_valve'], HEAT_PUMP['fan'], HEAT_PUMP['compressor']],
         'aux': [HEAT_PUMP['fan'], HEAT_PUMP['compressor'], HEAT_PUMP['aux']],
         'boost': [HEAT_PUMP['aux']],
         'emergency': [HEAT_PUMP['fan'], HEAT_PUMP['aux']],
@@ -79,12 +79,14 @@ class HestiaPi(Sensor):
         # save boost state
         self._boosting_heat = HVAC.OFF
         self._boosting_start_time = None
+        self._boosting_switch = None
 
         self.setup()
 
     def setup(self):
         logging.debug('Setting up HestiaPi')
         self.bme280 = BME280(self.name, self.topic)
+        self._boosting_switch = HVACAuxSwitch(self.name, None, self.topic)
 
         for mode, pins in HVAC.HEAT_PUMP_MODES.items():
             switch = BasicSwitch(self.name, pins, '{}_{}'.format(self.topic, mode), mode)
@@ -416,25 +418,39 @@ class HestiaPi(Sensor):
     def boost_heat(self, boost):
         # manually switching to AUX heat since we don't want to trigger state or mode safety checks.
         # self.mode = HVAC.AUX
-        if boost == HVAC.ON:
-            self.heat_boost_on()
-            self._boosting_start_time = pendulum.now()
-            # self._boosting_heat = HVAC.ON
-        elif boost == HVAC.OFF:
-            self.heat_boost_off()
-            # reset boost metadata
-            self._boosting_start_time = None
-            # self.temperature_history = []
-            # self._boosting_heat = HVAC.OFF
-        else:
-            raise HvacException('{} is not a valid boost value. allowed values are [[{},{}]'.format(boost, HVAC.ON, HVAC.OFF))
+        if self._boosting_switch.state == HVAC.ON:
+            if boost == HVAC.ON:
+                self.heat_boost_on()
+                self._boosting_start_time = pendulum.now()
+                # self._boosting_heat = HVAC.ON
+            elif boost == HVAC.OFF:
+                self.heat_boost_off()
+                # reset boost metadata
+                self._boosting_start_time = None
+                # self.temperature_history = []
+                # self._boosting_heat = HVAC.OFF
+            else:
+                raise HvacException('{} is not a valid boost value. allowed values are [[{},{}]'.format(boost, HVAC.ON, HVAC.OFF))
         # self._boosting_heat = boost
+        else:
+            logging.info("Boosting disabled by switch.")
     
+    def mqtt_ping(self, topic, callback):
+        logging.debug("Checing subcription status on topic {}".format(topic))
+        response = mqtt.publish(topic, "ping")
+        if response != 'pong':
+            logging.warn("Not subscribed to topic {}. Resubscribing...".format(topic))
+            mqtt.subscribe(topic, callback)
 
+    """
+    MQTT subscription callbacks
+    """
+    @MQTT.pongable
     def mqtt_set_temperature_set_point_callback(self, client, userdata, message):
         try:
+            payload = message.payload.decode()
             logging.info("Received temperature set point update request: {}".format(message.payload))
-            payload = float(message.payload.decode())
+            payload = float(payload)
             if self.mode == HVAC.HEAT:
                 self.set_point_heat = payload
             else:
@@ -442,8 +458,9 @@ class HestiaPi(Sensor):
         except Exception as e:
             logging.error('Unable to proces message.', e)
 
-        mqtt.publish(self.topic, self.payload())
+        MQTT.publish(self.topic, self.payload())
 
+    @MQTT.pongable
     def mqtt_set_fan_state_callback(self, client, userdata, message):
         try:
             payload = message.payload.decode().lower()
@@ -452,8 +469,9 @@ class HestiaPi(Sensor):
         except Exception as e:
             logging.error('Unable to proces message.', e)
 
-        mqtt.publish(self.topic, self.payload())
+        MQTT.publish(self.topic, self.payload())
 
+    @MQTT.pongable
     def mqtt_set_mode_callback(self, client, userdata, message):
         try:
             payload = message.payload.decode().lower()
@@ -462,19 +480,40 @@ class HestiaPi(Sensor):
         except Exception as e:
             logging.error('Unable to proces message.', e)
 
-        mqtt.publish(self.topic, self.payload())
+        MQTT.publish(self.topic, self.payload())
 
+    @MQTT.pongable
     def mqtt_set_aux_mode_callback(self, client, userdata, message):
         try:
             payload = message.payload.decode().lower()
             logging.info("Received aux mode update request: {}".format(payload))
-            # self.mode(payload)
             self.boost_heat(payload)
         except Exception as e:
             logging.error('Unable to proces message.', e)
 
-        mqtt.publish(self.topic, self.payload())
+        MQTT.publish(self.topic, self.payload())
 
 
+class HVACAuxSwitch(Switch):
 
+    def __init__(self, name, pin, topic, device_class='switch', device_type='generic_switch'):
+        super().__init__(name, pin, topic, device_class, device_type)
+
+    def setup(self, lazy_setup=True):
+        # return super().setup(lazy_setup)
+        # don't need any setup
+        self.on()
+
+    def setup_output(self):
+        # don't need to setup GPIO
+        pass
+
+    def on(self):
+        self.power_state = HVAC.ON
+
+    def off(self):
+        self.power_state = HVAC.OFF
+
+    def state(self):
+        return self.power_state
     
\ No newline at end of file
